

set -ex



which conda-build
conda-build -h
which conda-convert
conda-convert -h
which conda-develop
conda-develop -h
which conda-index
conda-index -h
which conda-inspect
conda-inspect linkages -h \| grep "--name ENVIRONMENT"
conda-inspect -h
which conda-metapackage
conda-metapackage -h
which conda-render
conda-render -h
which conda-skeleton
conda-skeleton -h
python test_bdist_conda_setup.py bdist_conda --help
exit 0
